/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Execute arbitrary system commands
 * 
 * Copyright (C) 1999, Mark Spencer
 *
 * Mark Spencer <markster@linux-support.net>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/callerid.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <pthread.h>


static char *tdesc = "Generic System() application with CID capability";

static char *app = "SystemCID";

static char *synopsis = "Execute a system command, after filling in the CID value";

static char *descrip =
"  SystemCID(command): First sprintf's the command with the CID, to fill in any %%s values.\n"
" Then, executes the resulting command  by  using  system(). Always Returns 0.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int systemcid_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char commandbuf[4096], callerid[150], *l, *n;
	
	if (!data) {
		ast_log(LOG_WARNING, "System requires an argument(command)\n");
		return -1;
	}
	LOCAL_USER_ADD(u);
	/* Do our thing here */
	if( chan->callerid )
		strncpy(callerid, chan->callerid, sizeof(callerid));
	else
		strcpy(callerid,"");
	ast_callerid_parse(callerid, &n, &l);
	if( l )
	{
		ast_shrink_phone_number(l);
	}
	else
	{
		l = "";
	}
	
	snprintf(commandbuf,4096,data,l);
	res = system(commandbuf);
	if (res != 0) {
		ast_log(LOG_WARNING, "Command '%s' returned %d\n", commandbuf, res);
		res = 0 /* -1 */;
	}
	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, systemcid_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
